//-------------------------------------------------------------------
//	File:		main.c
//
// Written By:		Nicola Ferrarese						
// This code implements a bipolar stepper motor control with a L297 L298 couple.
// See http://www.otocoup.com/index_e.htm to get schematic and PCB Layout
// PIC12F675 is used to generate STEP and DIR signals.
// The code was compiled using the HI-TECH PICC compiler v. 8.02PL1 Demo.
// Project > Build Options... > Project > PICC Compiler, check Enable assembler optimization 
// 
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	HEADER FILE INCLUDES
//---------------------------------------------------------------------
#include <pic12f6x.h>		// Register definitions
#include <pic.h>			// Header file 
#include "delay.h"			// Delay software routine 
#include "com_types.h"		// Common type definitions

#define	rpmmax	30600							// 30600	eq. 306.00 rpm
#define	rpmmin	600								// 600		eq.   6.00 rpm
#define	STEPREV	200								// motor's STEP number per revolution
#define velocityslope	(rpmmax-rpmmin)/120		// 120 is ADC Span for one direction (CC,CCW)=(0..120,135..255)

//	rps=rpm/60;									// revolution per second
// 	Ts=1/rps=60/rpm								// Period of a revolution in sec.
//	Tus=10^6 Ts=(10^6)*60/rpm					// Period of revolution in us
//	Tw=Tus/STEP=(10^6)*60/(rpm*STEPREV)			// Period wave in us
//  Please note that rpm variables are *100 value: Ex. rpm=10000 => 100.00 revolution per minute 
//	
//	Tw=(10^6)*60/[rpm/100)*STEP]=600000000/REVSTEP
// 	Ex. STEPREV=200 => rpm2us=30000000
#define	rpm2us	30000000						// rpm to STEP period conversion constant

//---------------------------------------------------------------------
//	Macros
//---------------------------------------------------------------------
__CONFIG(UNPROTECT & BORDIS & MCLREN & PWRTEN & WDTEN & INTIO);


//---------------------------------------------------------------------
//	Port Pin Designations
//---------------------------------------------------------------------
static bit STEP	@ BITNUM(GPIO, 1);
static bit DIR	@ BITNUM(GPIO, 2);
static bit LEDR	@ BITNUM(GPIO, 4);
static bit LEDV	@ BITNUM(GPIO, 5);


//---------------------------------------------------------------------
//	Variable declarations
//---------------------------------------------------------------------
static unsigned char
adc8;									// unsigned 8bit ADC result [0..255]

static unsigned int
rpm,									// rpmx100 ex. 18000->180.00
ts,										// step pulse lenght us
tck,									// 
timer1_init;							// timer1 value reloaded every interrput		

//---------------------------------------------------------------------
// Function Prototypes Dichiarazione
//---------------------------------------------------------------------			
static void SetUp(void);						// Initialize ports/peripherals 
static void InitVar(void);						// Initialize variable
static void interrupt Isr(void);				// Interrupt Service Routine
static unsigned char ADC_Measure8(void);		// Make 8bit ADC conversion
static unsigned int	calcrpm(unsigned char);		// Calculate rpm*100
static unsigned int calctw(unsigned int);		// Calculate STEP wave duration
static unsigned char startok(void);				// Check to system start with motor halded


//---------------------------------------------------------------------
// Main program
//---------------------------------------------------------------------
main()
{
SetUp();						// Initialize PORT and Peripheal
InitVar();						// Initialize variabile
while (startok());				// system starts only if potentiometer's cursor is on mid position	(rpm=0)			

while(1)										// This is the main program loop 
	{
	CLRWDT();
	if(T0IF)									// Polling tmr0 overflow
		{										// every 256x256us timer0 rolloff,256 prescler
		adc8=ADC_Measure8();					// measure set froma ext. potemtiometer
		rpm=calcrpm(adc8);						// calculate rpm profile
		if(rpm==0)
			{									// Motor halted
			TMR1IE=0;							// disable timer1 interrupt
			timer1_init=0;
			LEDR=LEDOFF;
			}
		else
			{									// Motor run						
			ts=calctw(rpm);						// calculate STEP wave pulse width
			tck=ts/4;							// half step/2; negative edge clock/2													
			timer1_init=0xFFFF-tck;				// calculate preset for timer1
			TMR1IE=1;							// Enable timer1 overflow interrupt
			LEDR=LEDON;
			}
		LEDV=!LEDV;								// heart beat
		T0IF=0;									// reset TMR0 interrupt flag
		}										//
	}											// end while
}												// end main										

	
//---------------------------------------------------------------------
//	Setup() initializes peripheral registers
//---------------------------------------------------------------------
static void SetUp(void)
{
						// SetUp I/O Port configuration 0=Output, 1=Input
TRISIO = 0x01;			// GP0 is analog input, others pins are outputs
GPIO   = 0x00;			// All pins Low
	
						// CMCON Comparator Control register
CM0=1;					// comparator off
CM1=1;
CM2=1;
						// ANSEL Analog select register
ANS3=0;					// 
ANS2=0;					//
ANS1=0;					// 
ANS0=1;					// GP0 analog input

						// ADCON0 AD control register
ADON=1;					// left justified, ch0, turn on adc module
						
						// OPTION_REG option register
PSA=0;					// GPIO pull-ups disabled,  Prescaler assigned to TMR0 module, 1:256
T0CS=0;					// TMRO internal CLK source,

						// SetUp interrupt
TMR1L=0x00;				// Reset Timer1 register pair
TMR1H=0x00;				// Reset Timer1 register pair
TMR1ON=1;				// Turn Timer1 on
PEIE=1;					// Enable all un masked peripheral interrupts
GIE=1;					// Enable Global Interrupts
}


//---------------------------------------------------------------------
//	InitVar() initializes program variables 
//---------------------------------------------------------------------
static void InitVar(void)
{
LEDR=LEDON;
LEDR=LEDOFF;
DIR=0;
STEP=0;
}


//---------------------------------------------------------------------
//	Isr() Interrupt service routine
//---------------------------------------------------------------------
static void interrupt Isr(void)
{										// save status, bit overhead						
TMR1L=timer1_init&0x00FF;				// Load 16 bit tmr1 value 
TMR1H=(unsigned int)timer1_init>>8;		// 

STEP=!STEP;								// 
TMR1IF=0; 								// clear Timer1 interrupt Flag
}										// end Isr



//---------------------------------------------------------------------
//	8bit Analog to Digital conversion
//---------------------------------------------------------------------
static unsigned char ADC_Measure8(void)
{
GODONE = 1;  	                    // start ADC conversion
while(GODONE) {};					// Wait end of conversion
ADIE=0;				
return ADRESH;						// 8bit result, left justified
}



//---------------------------------------------------------------------
//	Calculate rpm*100 and set direction
//---------------------------------------------------------------------
static unsigned int	calcrpm(unsigned char adc8)
{
// return rpm value*100 calculated on adc8 readed value;
// table below are implemented: 
// adc8=0 		-> rpmmax	DIR=0
// adc8=120 		-> rpmmin	DIR=0
// 120<adc8<135	-> rpm=0
// adc8=135		-> rpm_min	DIR=1
// adc8=255		-> rpmmax	DIR=1
//
unsigned int retval;
if(adc8<121) 
	{
	DIR=0;
	retval=(unsigned int) (rpmmin+velocityslope*(120-adc8));
	}
else
	{
	if(adc8>134)
		{
		DIR=1;
		retval=(unsigned int) (rpmmin+velocityslope*(adc8-135));
		}
	else retval=0;
	}
return retval;
}


//---------------------------------------------------------------------
//	Calculate Twait [us]
//---------------------------------------------------------------------
static unsigned int calctw(unsigned int rpm)
{
// convert rpm to a time lenght for a STEP controll
// an [us] value are returned.
unsigned int retval;
if(rpm) retval=(unsigned long) (rpm2us/rpm);
else retval=0xFFFF;
return retval;
}
	

//---------------------------------------------------------------------
//	Check to Start with motor halted
//---------------------------------------------------------------------
static char startok(void)
{
char retval=0;
adc8=ADC_Measure8();
if((adc8<121)||(adc8>134))
	{
	DelayMs(10);
	CLRWDT();
	LEDV=!LEDV;								// heart beat
	retval=255;
	}
return retval;
}

 







